@@ -19,9 +19,7 @@ USA.*/
 /* Routines for basic matrix manipulation creation, destruction and file i/o
    for matrices. See end of file for update log */
 
-
- 
-
+#include <R.h>
 #include <stdio.h>
 #include <math.h>
 #include <stdlib.h>
@@ -68,16 +66,16 @@ matrix initmat(rows,cols) long rows,cols;
   pad=0L;
 #endif
   A.vec=0;
-  A.M=(double **)calloc((size_t)(rows+2*pad),sizeof(double *));
+  A.M=(double **)R_chk_calloc((size_t)(rows+2*pad),sizeof(double *));
   if ((cols==1L)||(rows==1L))
   { if (A.M)
-    A.M[0]=(double *)calloc((size_t)(cols*rows+2*pad),sizeof(double));
+    A.M[0]=(double *)R_chk_calloc((size_t)(cols*rows+2*pad),sizeof(double));
     for (i=1L;i<rows+2*pad;i++)
     A.M[i]=A.M[0]+i*cols;A.vec=1;
   } else
   { if (A.M)
     for (i=0L;i<rows+2*pad;i++)
-    A.M[i]=(double *)calloc((size_t)(cols+2*pad),sizeof(double));
+    A.M[i]=(double *)R_chk_calloc((size_t)(cols+2*pad),sizeof(double));
   }
   A.mem=rows*cols*sizeof(double);
   memused+=A.mem;matrallocd++;
@@ -104,10 +102,10 @@ matrix initmat(rows,cols) long rows,cols;
     A.V=A.M[0];
   /* putting a record of the matrix on the linked list of all extant matrices */
     if (matrallocd==1) /*new list*/
-    { top=bottom=(MREC *)calloc(1,sizeof(MREC));
+    { top=bottom=(MREC *)R_chk_calloc(1,sizeof(MREC));
       bottom->mat=top->mat=A;top->bp=bottom;bottom->fp=top;
     } else  /* expanding the linked list by one */
-    { top->fp=(MREC *)calloc(1,sizeof(MREC));
+    { top->fp=(MREC *)R_chk_calloc(1,sizeof(MREC));
       top->fp->mat=A;top->fp->bp=top;top=top->fp; /* crystal clear, no? */
     }
   }
@@ -159,16 +157,16 @@ void freemat(A) matrix A;
 	     if (i!=matrallocd-1)
 	     delet->fp->bp=delet->bp;
 	     else top=delet->bp;
-	     free(delet);
+	     R_chk_free(delet);
       }
       /* repositioning pointers so that what was allocated gets freed */
       if (!A.vec) for (i=0;i<pad;i++) A.M--;
       for (i=0;i<A.original_r+2*pad;i++)
       for (j=0;j<pad;j++) A.M[i]--;
     }
-    if (A.vec) free(A.M[0]); else
-    for (i=0;i<A.original_r+2*pad;i++) if (A.M[i]) free(A.M[i]);
-    if (A.M) free(A.M);
+    if (A.vec) R_chk_free(A.M[0]); else
+    for (i=0;i<A.original_r+2*pad;i++) if (A.M[i]) R_chk_free(A.M[i]);
+    if (A.M) R_chk_free(A.M);
     memused -= A.mem;matrallocd--;
   }
 }
@@ -346,8 +344,8 @@ void multi(int n,matrix C,...)
 #else
   va_start(argptr,C);
 #endif
-  t=(int *)calloc(n,sizeof(int));
-  M=(matrix *)calloc(n,sizeof(matrix)); /* previously no cast for sun C ??! */
+  t=(int *)R_chk_calloc((size_t)n,sizeof(int));
+  M=(matrix *)R_chk_calloc((size_t)n,sizeof(matrix)); /* previously no cast for sun C ??! */
   for (i=0;i<n;i++) M[i]=/*(matrix)*/ va_arg(argptr,matrix);
   for (i=0;i<n;i++) t[i]=(int) va_arg(argptr,int);
   if (t[0]) r=M[0].c; else r=M[0].r;
@@ -365,8 +363,8 @@ void multi(int n,matrix C,...)
   { matmult(C,temp0,M[n-1],0,t[n-1]);
     freemat(temp0);
   }
-  free(t);
-  free(M);
+  R_chk_free(t);
+  R_chk_free(M);
   va_end(argptr);
 }
 
@@ -383,10 +381,10 @@ void invert(matrix *A)
 { double **AM,*p,*p1,max,x;
   int *c,*rp,*cp,i,j,k,pr=0,pc=0,*d,cj,ck;
   if (A->r!=A->c) ErrorMessage(_("Attempt to invert() non-square matrix"),1);
-  c=(int *)calloc((size_t)A->c,sizeof(int)); /* index of columns, used for column pivoting */
-  d=(int *)calloc((size_t)A->c,sizeof(int));
-  rp=(int *)calloc((size_t)A->c,sizeof(int)); /* row changes */
-  cp=(int *)calloc((size_t)A->c,sizeof(int)); /* row changes */
+  c=(int *)R_chk_calloc((size_t)A->c,sizeof(int)); /* index of columns, used for column pivoting */
+  d=(int *)R_chk_calloc((size_t)A->c,sizeof(int));
+  rp=(int *)R_chk_calloc((size_t)A->c,sizeof(int)); /* row changes */
+  cp=(int *)R_chk_calloc((size_t)A->c,sizeof(int)); /* row changes */
   for (i=0;i<A->c;i++) { c[i]=i;d[i]=i;}
   AM=A->M;            /* saving adress calculations*/
   for (j=0;j<A->c;j++) /* loop through columns to be reduced */
@@ -441,7 +439,7 @@ void invert(matrix *A)
     { p=AM[k];x=p[i];p[i]=p[rp[i]];p[rp[i]]=x;} /* column exchange  */
   }
    
-  free(c);free(rp);free(cp);free(d);
+  R_chk_free(c);R_chk_free(rp);R_chk_free(cp);R_chk_free(d);
 }
 
 void tricholeski(matrix *T,matrix *l0,matrix *l1)
@@ -811,7 +809,7 @@ int QR(matrix *Q,matrix *R)
   double *u,t,z,**RM,*p,m;
   RM=R->M;Rr=R->r;
   if (Rr<R->c) n=Rr; else n=R->c;
-  u=(double *)calloc((size_t)Rr,sizeof(double));
+  u=(double *)R_chk_calloc((size_t)Rr,sizeof(double));
   for (k=0;k<n;k++)
   { m=0.0;for (i=k;i<Rr;i++) { z=RM[i][k];z=fabs(z);if (z>m) m=z;}
     if (m) for (i=k;i<Rr;i++) RM[i][k]/=m; /* avoid over/underflow problems */
@@ -823,7 +821,7 @@ int QR(matrix *Q,matrix *R)
     t=t*t;t+=u[k]*u[k]-z*z; /* efficient t calculation */
     /* t=0.0;for (p=u+k;p<u+Rr;p++) t+= *p * *p; - old inefficient calculation */
     t=sqrt(t/2);
-    if (t==0.0) {free(u);return(0);} /* singular matrix */
+    if (t==0.0) {R_chk_free(u);return(0);} /* singular matrix */
     for (p=u+k;p<u+Rr;p++) *p /= t;
     for (j=k+1;j<R->c;j++)
     { t=0.0;for (i=k;i<Rr;i++) t+=u[i]*RM[i][j];
@@ -834,7 +832,7 @@ int QR(matrix *Q,matrix *R)
       for (i=k;i<Rr;i++) p[i]=u[i];
     }
   }
-  free(u);
+  R_chk_free(u);
   return(1);
 }
 
@@ -1438,15 +1436,15 @@ void sort(matrix a)
 
 
 int real_elemcmp(const void *a,const void *b,int el)
-
+/* declaring this inline static slows it down!! */
 { static int k=0;
-  int i;
-  double *na,*nb;
-  if (el>=0) { k=el;return(0);}
+  double *na,*nb,*nak;
+  if (el>0) { k=el;return(0);}
   na=(*(double **)a);nb=(*(double **)b);
-  for (i=0;i<k;i++) 
-  { if (na[i]<nb[i]) return(-1);
-    if (na[i]>nb[i]) return(1);
+  nak = na + k;
+  for (;na<nak;na++,nb++) { 
+    if (*na < *nb) return(-1);
+    if (*na > *nb) return(1);
   }
   return(0);
 }
